DEFAULT_USER_AGENT = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36"
XPATH_TEXT = "//comment() | //text()[not (ancestor-or-self::script or ancestor-or-self::style)]"
XPATH_TEXT_INCLUDE_JAVASCRIPT = "//comment() | //text()[not (ancestor-or-self::style)]"
XPATH_TEXT_INCLUDE_CSS = "//comment() | //text()[not (ancestor-or-self::script)]"
XPATH_TEXT_INCLUDE_JAVASCRIPT_AND_CSS = "//comment() | //text()"
XPATH_COMMENT = "//comment()"
CHARACTERS_TO_FILTER_AWAY = '[\(\),./"\"?!“”‘’´:\{\}\[\]«»\*…•‹≈=■◦☀️„|_~✓+<>@;]'
CHARACTERS_ALLOWED_IN_WORDS_BUT_NOT_IN_START_OR_END = "'ˈ-–━—&"



import html
import re
import scrapy
import scrapy.exceptions
import tld
from scrapy import signals
from scrapy.linkextractors import LinkExtractor
from scrapy.spidermiddlewares import offsite
from scrapy.spiders import CrawlSpider, Rule
import scrapy.core.downloader.handlers.http

try:
    from . import constants
except ImportError:  # When running this file directly
    import os
    import sys
    sys.path.insert(0, os.path.abspath('.'))
    import constants


class OnlyExactSameDomainSpiderMiddleware(offsite.OffsiteMiddleware):

    def get_host_regex(self, spider):
        regex = super().get_host_regex(spider)
        # Remove optional .* (any subdomains) from regex
        regex = regex.pattern.replace("(.*\.)?", "", 1)
        return re.compile(regex)


class OnlyChildrenSubdomainAndSameDomainSpiderMiddleware(offsite.OffsiteMiddleware):

    def get_host_regex(self, spider):
        return super().get_host_regex(spider)


class AnyParentAndSisterAndSubdomainMiddleware(offsite.OffsiteMiddleware):

    has_altered_allowed_domains = False

    def get_host_regex(self, spider):
        if not self.has_altered_allowed_domains:
            self.has_altered_allowed_domains = True
            tld_obj = tld.get_tld(spider.start_urls[0], as_object=True, fail_silently=True)
            if tld_obj is None:
                # Typically localhost or similar - just keep spider.allowed_domains unaltered
                # TODO: Log something in this case? Or throw and force change of settings?
                pass
            else:
                spider.allowed_domains = [tld_obj.domain + "." + tld_obj.tld]
        regex = super().get_host_regex(spider)
        return re.compile(regex)


class CewlerSpider(CrawlSpider):
    name = "CeWLeR"

    def __init__(self, console, url, file_words=None, file_urls=None, include_js=False, include_css=False, should_lowercase=False, without_numbers=False, min_word_length=5, verbose=False, spider_event_callback=None, stream_to_file=False, *args, **kwargs):
        self.console = console
        self.should_lowercase = should_lowercase
        self.without_numbers = without_numbers
        self.min_word_length = min_word_length
        self.verbose = verbose
        self.spider_event_callback = spider_event_callback
        self.stream_to_file = stream_to_file
        self.words_found = set()
        self.urls_parsed = set()
        self.visited_domains = set()
        self.exceptions = []
        self.unsupported_content_types = set()
        self.log_lines = []
        self.last_status = "init"
        self.include_js = include_js
        self.include_css = include_css
        if self.include_js and self.include_css:
            deny_extensions = scrapy.linkextractors.IGNORED_EXTENSIONS
            deny_extensions.remove("css")
            self.link_extractor = LinkExtractor(tags=("a", "area", "script", "link"), attrs=("href", "src"), deny_extensions=deny_extensions)
            self.xpath = constants.XPATH_TEXT_INCLUDE_JAVASCRIPT_AND_CSS
        elif self.include_js:
            self.link_extractor = LinkExtractor(tags=("a", "area", "script"), attrs=("href", "src"))
            self.xpath = constants.XPATH_TEXT_INCLUDE_JAVASCRIPT
        elif self.include_css:
            deny_extensions = scrapy.linkextractors.IGNORED_EXTENSIONS
            deny_extensions.remove("css")
            self.link_extractor = LinkExtractor(tags=("a", "area", "link"), attrs=("href", "src"), deny_extensions=deny_extensions)
            self.xpath = constants.XPATH_TEXT_INCLUDE_CSS
        else:
            self.link_extractor = LinkExtractor()
            self.xpath = constants.XPATH_TEXT
        try:
            self.rules = (Rule(self.link_extractor, follow=True, callback="parse_item"),)
            super(CewlerSpider, self).__init__(*args, **kwargs)
            self.start_urls = [url]

            self.allowed_domains = [self.get_allowed(url)]
            self.file_words = open(file_words, mode="w") if file_words is not None else None
            self.file_urls = open(file_urls, mode="w") if file_urls is not None else None
        except Exception:
            self.console.print_exception(show_locals=False)
            raise scrapy.exceptions.CloseSpider()

    def parse_start_url(self, response):
        return self.parse_item(response)

    @classmethod
    def from_crawler(cls, crawler, *args, **kwargs):
        spider = super(CewlerSpider, cls).from_crawler(crawler, *args, **kwargs)
        crawler.signals.connect(spider.spider_closed, signal=signals.spider_closed)
        crawler.signals.connect(spider.request_reached_downloader, signal=signals.request_reached_downloader)
        # crawler.signals.connect(spider.signal,signal=signals.engine_started)
        # crawler.signals.connect(spider.signal,signal=signals.item_scraped)
        crawler.signals.connect(spider.engine_stopped, signal=signals.engine_stopped)
        return spider

    def get_allowed(self, url):
        #print("get_allowed", url)
        return re.findall("^(?:https?:\/\/)?(?:[^@\/\n]+@)?([^:\/\n]+)", url)[0]

    def send_spider_callback(self):
        if self.spider_event_callback is not None:
            self.spider_event_callback(
                {
                    "stats": self.crawler.stats.get_stats(),
                    "words": self.words_found,
                    "domains": self.visited_domains,
                    "status": self.last_status,
                    "unsupported_content_types": self.unsupported_content_types,
                    "exceptions": self.exceptions
                }
            )

    def request_reached_downloader(self, request, spider):
        self.last_status = ("request_reached_downloader", request.url)
        self.send_spider_callback()

    def engine_stopped(self):
        self.last_status = "engine_stopped"
        self.send_spider_callback()

    def spider_closed(self, spider):
        self.words_found = sorted(self.words_found)
        if not self.stream_to_file:
            if self.file_words is not None:
                self.last_status = "writing_to_file"
                for word in self.words_found:
                    self.file_words.write(word)
                    self.file_words.write("\n")
                self.file_words.close()
            if self.file_urls is not None:
                self.last_status = "writing_to_file"
                for url in sorted(self.urls_parsed):
                    self.file_urls.write(url)
                    self.file_urls.write("\n")
                self.file_urls.close()
        self.last_status = "spider_closed"
        self.send_spider_callback()

    def _get_words_from_text(self, text):
        new_words = []
        text = html.unescape(text)
        text = re.sub(constants.CHARACTERS_TO_FILTER_AWAY, " ", text)  # Filter characters
        text = re.sub("\s+", " ", text)  # Replace all spacing with one space
        if self.should_lowercase:
            text = text.lower()
        for word in text.split(" "):
            word = word.strip()
            while True:
                found = False
                for c in constants.CHARACTERS_ALLOWED_IN_WORDS_BUT_NOT_IN_START_OR_END:
                    if word.startswith(c):
                        word = word[1:]
                        found = True
                    if word.endswith(c):
                        word = word[:-1]
                        found = True
                if not found:
                    break
            if len(word) >= max(1, self.min_word_length):
                if self.without_numbers:
                    if not bool(re.search(r'\d', word)):
                        new_words.append(word)
                else:
                    new_words.append(word)
        return set(new_words)

    def _get_words_from_text_response(self, response):
        new_words = set()
        try:
            new_words = self._get_words_from_text(response.text)
            if len(new_words) > 0:
                if self.file_words is not None and self.stream_to_file:
                    for word in new_words:
                        if word not in self.words_found:
                            self.words_found.add(word)
                            self.file_words.write(word + "\n")
                    self.file_words.flush()
                else:
                    self.words_found.update(new_words)
        except Exception as e:
            exit(e)
        return new_words

    def _get_words_from_html_response(self, response):
        new_words = set()
        try:
            for item in response.xpath(self.xpath):
                new_words.update(self._get_words_from_text(item.get()))
            if len(new_words) > 0:
                if self.file_words is not None and self.stream_to_file:
                    for word in new_words:
                        if word not in self.words_found:
                            self.words_found.add(word)
                            self.file_words.write(word + "\n")
                    self.file_words.flush()
                else:
                    self.words_found.update(new_words)
        except Exception as e:
            exit(e)
        return new_words

    def parse_item(self, response):
        try:
            if self.stream_to_file and self.file_urls is not None:
                if response.url not in self.urls_parsed:
                    self.urls_parsed.add(response.url)
                    self.file_urls.write(response.url + "\n")
                self.file_urls.flush()
            else:
                self.urls_parsed.add(response.url)
            headers = {key.lower(): value for key, value in response.headers.items()}
            if (b"content-type" in headers.keys()):
                tld_obj = tld.get_tld(response.url, as_object=True, fail_silently=True)
                if tld_obj is not None:
                    self.visited_domains.add(tld_obj.subdomain + "." + tld_obj.domain + "." + tld_obj.tld)
                content_type = headers[b"content-type"][0]
                if (b"text/html" in content_type):
                    yield {
                        "url": response.url,
                        "title": response.css("title::text").get(),
                        "words:": self._get_words_from_html_response(response),
                    }
                    for item in response.xpath(constants.XPATH_COMMENT):
                        inside_comment = item.get().replace("<!--", "").replace("-->", "")
                        for link in self.link_extractor.extract_links(scrapy.http.TextResponse(url=response.url, body=bytes(inside_comment, encoding="utf-8"))):
                            yield response.follow(link, callback=self.parse_item)
                elif b"text/plain" in content_type:
                    yield {
                        "url": response.url,
                        "title": "",
                        "words:": self._get_words_from_text_response(response),
                    }
                elif self.include_js and (b"script" in content_type or b"application/json" in content_type or b"application/ld+json" in content_type):
                    yield {
                        "url": response.url,
                        "title": "",
                        "words:": self._get_words_from_text_response(response),
                    }
                elif self.include_css and (b"text/css" in content_type):
                    yield {
                        "url": response.url,
                        "title": "",
                        "words:": self._get_words_from_text_response(response),
                    }
                else:
                    try:
                        self.unsupported_content_types.add(content_type.decode("utf-8"))
                    except UnicodeDecodeError:
                        self.unsupported_content_types.add(str(content_type))
                    yield {
                        "url": response.url,
                        "title": "",
                        "error": f"Unsupported Content-Type [{b'content-type' in headers.keys()}]"
                    }
            else:
                yield {
                    "url": response.url,
                    "title": "",
                    "error": f"Skipping because of Content-Type [{b'content-type' in headers.keys()}]"
                }
        except Exception as e:
            self.exceptions.append(e)
            yield {"url": response.url, "title": "", "error": str(e)}



#!/usr/bin/python3
import argparse
import logging
import math
import sys
import textwrap
import time

from rich import print
from rich.console import *
from rich.console import Console
from rich.live import Live
from rich.padding import Padding
from rich.panel import Panel
from rich.table import Table
from scrapy.crawler import CrawlerProcess

try:
    from . import constants
    from . import spider
    spider_package = "cewler."
except ImportError:  # When running this file directly
    sys.path.insert(0, os.path.abspath('.'))
    import constants
    import spider
    spider_package = ""


__author__ = "Roy Solberg"
__version__ = "1.0.8"
__program__ = "CeWLeR"
__description__ = "Custom Word List generator Redefined"

start_time = time.time()
live = None
static_ui_lines = None
console = None
longest_status_text = 0
is_verbose_output = False
last_event_received = None


def insert_dev_arguments():
    """ Used mainly to easily access options during development """
    if len(sys.argv) == 1:
        # sys.argv.append("https://roysolberg.com")
        sys.argv.append("http://localhost:8000")
        sys.argv.append("-o")
        sys.argv.append("wordlist-dev.txt")
        sys.argv.append("--output-urls")
        sys.argv.append("urls-dev.txt")
        sys.argv.append("--subdomain_strategy")
        sys.argv.append("all")
        # sys.argv.append("children")
        # sys.argv.append("exact")
        sys.argv.append("--depth")
        sys.argv.append("0")
        sys.argv.append("--include-js")
        sys.argv.append("--include-css")
        # sys.argv.append("--min-word-length")
        # sys.argv.append("5")
        # sys.argv.append("--stream")
        # sys.argv.append("--lowercase")
        # sys.argv.append("--user-agent")
        # sys.argv.append(f"{__program__} v.{__version__} - {__description__}")
        # sys.argv.append("--rate")
        # sys.argv.append("1")
        # sys.argv.append("--without-numbers")
        sys.argv.append("--verbose")


def get_parsed_args_and_init_parser():
    parser = argparse.ArgumentParser(prog=__program__.lower(), allow_abbrev=False, description=__description__, epilog="Visit https://github.com/roys/cewler for more information")

    parser.add_argument("url", help="URL to start crawling from")
    parser.add_argument("-d", "--depth", type=int, default=2, help="directory path depth to crawl, 0 for unlimited (default: 2)")
    parser.add_argument("-css", "--include-css", action="store_true", help="include CSS from external files and <style> tags")
    parser.add_argument("-js", "--include-js", action="store_true", help="include JavaScript from external files and <script> tags")
    parser.add_argument("-l", "--lowercase", action="store_true", help="lowercase all parsed words")
    parser.add_argument("-m", "--min-word-length", type=int, default=5)
    parser.add_argument("-o", "--output", help="file were to stream and store wordlist instead of screen (default: screen)")
    parser.add_argument("-ou", "--output-urls", help="file were to stream and store URLs visited (default: not outputted)")
    parser.add_argument("-r", "--rate", type=int, default=20, help="requests per second (default: 20)")
    parser.add_argument("-s", "--subdomain_strategy", choices=["all", "children", "exact"], default="exact", help="allow crawling [all] domains, including children and siblings, only [exact] the same (sub)domain (default), or same domain and any belonging [children]")
    parser.add_argument("--stream", action="store_true", default=False, help="writes to file after each request (may produce duplicates because of threading) (default: false)")
    parser.add_argument("-u", "--user-agent", default=constants.DEFAULT_USER_AGENT, help=f"User-Agent header to send (default: {constants.DEFAULT_USER_AGENT})")
    parser.add_argument("-v", "--verbose", action="store_true", help="A bit more detailed output")
    parser.add_argument("-w", "--without-numbers", action="store_true", help="ignore words are numbers or contain numbers")

    args = parser.parse_args(args=None if sys.argv[1:] else ['--help'])
    if args.stream and args.output is None:
        exit("cewler: error: Argument --stream cannot be used without a file specified with --output")
    return args


def get_scrapy_settings_and_init_logging(user_agent, depth_limit, reqs_per_sec, subdomain_strategy):
    """ Sets up scrapy logging and returns necessary settings object """
    logging.getLogger("scrapy").setLevel(logging.ERROR)
    logging.getLogger("scrapy").propagate = False

    if subdomain_strategy == "all":
        offsite_class = f"{spider_package}spider.AnyParentAndSisterAndSubdomainMiddleware"
    elif subdomain_strategy == "children":
        offsite_class = f"{spider_package}spider.OnlyChildrenSubdomainAndSameDomainSpiderMiddleware"
    else:  # "exact"
        offsite_class = f"{spider_package}spider.OnlyExactSameDomainSpiderMiddleware"

    middleware_settings = {
        "scrapy.spidermiddlewares.offsite.OffsiteMiddleware": None,
        offsite_class: 1337
    }

    return {
        # https://docs.scrapy.org/en/latest/topics/settings.html
        "REQUEST_FINGERPRINTER_IMPLEMENTATION": "2.7",
        "USER_AGENT": user_agent,
        "DEPTH_LIMIT": depth_limit,
        "DOWNLOAD_DELAY": 1/reqs_per_sec,
        "SPIDER_MIDDLEWARES": middleware_settings,
        "HTTPERROR_ALLOW_ALL": True
        # "CONCURRENT_REQUESTS": 16
        # "CONCURRENT_REQUESTS_PER_DOMAIN": 8
        # "CONCURRENT_REQUESTS_PER_IP": 0
        # "CONCURRENT_REQUESTS": 16
    }


def on_spider_event(event):
    """ Callback from spider with stats and other events """
    global last_event_received
    last_event_received = event
    if live is not None:
        live.update(generate_ui())


def get_nice_bytes(bytes=None):
    if bytes == None:
        return "-"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(bytes, 1024)))
    p = math.pow(1024, i)
    s = round(bytes / p, 2)
    return "%s %s" % (s, size_name[i])


def get_live_ui(args):
    if args.subdomain_strategy == "all":
        nice_strategy = "Any and all subdomains of top domain"
    elif args.subdomain_strategy == "children":
        nice_strategy = "Same domain + any child subdomains"
    else:  # "exact"
        nice_strategy = "Exact same domain"
    nice_strategy += f", max depth {args.depth}"
    nice_strategy += f", {args.rate} reqs/s"
    nice_words = "Lowercase" if args.lowercase else "Mixed case"
    nice_words += ", " + ("excl. numbers" if args.without_numbers else "incl. numbers")
    nice_words += f", min. {args.min_word_length} chars."
    nice_ua = "Default" if constants.DEFAULT_USER_AGENT == args.user_agent else "Custom"
    nice_ua += " (" + textwrap.shorten(args.user_agent, width=40, placeholder="...") + ")"
    nice_output = "Screen only" if args.output is None else args.output

    global is_verbose_output
    is_verbose_output = args.verbose
    global static_ui_lines
    static_ui_lines = []
    static_ui_lines.append(["URL: ", f"[bold underline blue]{args.url}"])
    static_ui_lines.append(["Strategy: ", f"[magenta]{nice_strategy}"])
    static_ui_lines.append(["Words: ", f"[magenta]{nice_words}"])
    static_ui_lines.append(["User-Agent: ", f"[magenta]{nice_ua}"])
    static_ui_lines.append(["Output: ", f"[magenta]{nice_output}"])

    print("\n")
    return Live(generate_ui(), console=console, refresh_per_second=4, auto_refresh=True)


def generate_ui():
    try:
        grid = Table.grid(expand=True)
        for row in static_ui_lines:
            grid.add_row(row[0], row[1])

        event = last_event_received
        if is_verbose_output and event is not None and "status" in event:
            status = event["status"]
            color = "[green]"
            if isinstance(status, str):
                if status == "init":
                    status = "Initializing..."
                elif status == "writing_to_file":
                    status = "Writing words to file..."
                elif status == "spider_closed":
                    status = "Spider stopped"
                elif status == "engine_stopped":
                    status = "Spider engine stopped"
                status_length = len(status)
            elif isinstance(status, tuple):
                if status[0] == "request_reached_downloader":
                    status = f"Requesting [underline blue]{status[1]}[/underline blue]"
                    status_length = len(status)
            global longest_status_text
            if status_length > longest_status_text:
                longest_status_text = status_length
            status = status.ljust(longest_status_text, " ")
            grid.add_row("Status: ", color + status)

        time_spent = time.time() - start_time
        panel_footer = None if time_spent < 60 else "Press <ctrl> + c [bold]once[/bold] to abort and write to file"
        hours, remainder = divmod(time_spent, 3600)
        minutes, seconds = divmod(remainder, 60)
        if hours == 0:
            time_spent = "[magenta]{:02}:{:02}".format(int(minutes), int(seconds))
        else:
            time_spent = "[magenta]{:02}:{:02}:{:02}".format(int(hours), int(minutes), int(seconds))
        grid.add_row("Time elapsed: ", time_spent)

        if event is not None and "stats" in event and "downloader/request_count" in event["stats"] and "downloader/response_count" in event["stats"]:
            requests = "[magenta]" + str(event["stats"]["downloader/response_count"]) + "/" + str(
                event["stats"]["downloader/request_count"])
            if "domains" in event and len(event["domains"]) > 1:
                requests += f" @ {len(event['domains'])} domains"
        else:
            requests = "[magenta]-"
        grid.add_row("Requests: ", requests)

        if event is not None and "stats" in event and "downloader/response_bytes" in event["stats"]:
            received = event["stats"]["downloader/response_bytes"]
        else:
            received = None
        grid.add_row("Data received: ", "[magenta]" + get_nice_bytes(received))

        if is_verbose_output and event is not None and "unsupported_content_types" in event:
            unsupported_content_types = event["unsupported_content_types"]
            if (len(unsupported_content_types) > 0):
                grid.add_row("Unsupported: ", ", ".join(sorted(unsupported_content_types)))

        if event is not None and "words" in event:
            words = str(len(event["words"]))
        else:
            words = "-"
        grid.add_row("Words found: ", "[green bold]" + words)

        renderable = Panel.fit(Padding(grid, (1, 2)), title=f"{__program__} v.{__version__} - {__description__}", subtitle=panel_footer)

        if event is not None and "exceptions" in event:
            exceptions = event["exceptions"]
            if (len(exceptions) > 0):
                for exception in exceptions:
                    try:
                        raise exception
                    except Exception:
                        console.print_exception(show_locals=False)
                event["exceptions"].clear()

        return renderable
    except Exception:
        console.print_exception(show_locals=False)


def cewler():
    global console
    console = Console()
    try:
        # insert_dev_arguments()  # Used for development
        args = get_parsed_args_and_init_parser()
        global live
        live = get_live_ui(args)

        with live:
            process = CrawlerProcess(get_scrapy_settings_and_init_logging(args.user_agent, args.depth, args.rate, args.subdomain_strategy))
            process.crawl(spider.CewlerSpider, console=console, url=args.url, file_words=args.output, file_urls=args.output_urls, include_js=args.include_js, include_css=args.include_css, should_lowercase=args.lowercase,
                          without_numbers=args.without_numbers, min_word_length=args.min_word_length, verbose=args.verbose, stream_to_file=args.stream, spider_event_callback=on_spider_event)
            process.start()
        print("")

        if args.output is None:  # Should output to screen
            if last_event_received is not None and "words" in last_event_received:
                for word in last_event_received["words"]:
                    print(word)
                print("")
    except Exception:
        console.print_exception(show_locals=False)


if __name__ == "__main__":
    cewler()



import sys
sys.path.append("src/cewler")  # noqa (to avoid autopep8 organizing imports)
from cewler.spider import *


def test_OnlyExactSameDomainSpiderMiddleware():
    url = "http://example.com"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = OnlyExactSameDomainSpiderMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("example.com") != None
    assert pattern.match("roysolberg.com") == None
    assert pattern.match("2example.com") == None
    assert pattern.match("subdomain.example.com") == None
    assert pattern.match("sub.subdomain.example.com") == None

    url = "https://subdomain.example.com"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = OnlyExactSameDomainSpiderMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("example.com") == None
    assert pattern.match("roysolberg.com") == None
    assert pattern.match("2example.com") == None
    assert pattern.match("subdomain.example.com") != None
    assert pattern.match("sub.subdomain.example.com") == None


def test_OnlyChildrenSubdomainAndSameDomainSpiderMiddleware():
    url = "http://example.com"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = OnlyChildrenSubdomainAndSameDomainSpiderMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("example.com") != None
    assert pattern.match("roysolberg.com") == None
    assert pattern.match("2example.com") == None
    assert pattern.match("subdomain.example.com") != None
    assert pattern.match("sub.subdomain.example.com") != None

    url = "http://subdomain.example.com"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = OnlyChildrenSubdomainAndSameDomainSpiderMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("example.com") == None
    assert pattern.match("roysolberg.com") == None
    assert pattern.match("2example.com") == None
    assert pattern.match("other.example.com") == None
    assert pattern.match("subdomain.example.com") != None
    assert pattern.match("sub.subdomain.example.com") != None


def test_AnyParentAndSisterAndSubdomainMiddleware():
    url = "https://example.com"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = AnyParentAndSisterAndSubdomainMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("example.com") != None
    assert pattern.match("roysolberg.com") == None
    assert pattern.match("2example.com") == None
    assert pattern.match("other.example.com") != None
    assert pattern.match("subdomain.example.com") != None
    assert pattern.match("sub.subdomain.example.com") != None
    assert pattern.match("www.example.com") != None

    url = "http://subdomain.example.com"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = AnyParentAndSisterAndSubdomainMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("example.com") != None
    assert pattern.match("roysolberg.com") == None
    assert pattern.match("2example.com") == None
    assert pattern.match("other.example.com") != None
    assert pattern.match("subdomain.example.com") != None
    assert pattern.match("sub.subdomain.example.com") != None

    url = "https://google.co.uk"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = AnyParentAndSisterAndSubdomainMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("google.co.uk") != None
    assert pattern.match("bing.co.uk") == None
    assert pattern.match("2google.co.uk") == None
    assert pattern.match("subdomain.google.co.uk") != None
    assert pattern.match("sub.subdomain.google.co.uk") != None

    url = "https://supersub.google.co.uk"
    spider = CewlerSpider(Console(), url, file_words=None)
    mw = AnyParentAndSisterAndSubdomainMiddleware(stats=None)
    pattern = mw.get_host_regex(spider)
    assert pattern.match("google.co.uk") != None
    assert pattern.match("bing.co.uk") == None
    assert pattern.match("2google.co.uk") == None
    assert pattern.match("subdomain.google.co.uk") != None
    assert pattern.match("sub.subdomain.google.co.uk") != None


if __name__ == "__main__":
    test_OnlyExactSameDomainSpiderMiddleware()
    test_OnlyChildrenSubdomainAndSameDomainSpiderMiddleware()
    test_AnyParentAndSisterAndSubdomainMiddleware()
    print("Everything passed")
